'use strict';

var crypto = require('crypto');
crypto = 'default' in crypto ? crypto['default'] : crypto;
var rest = require('restler');
rest = 'default' in rest ? rest['default'] : rest;
var js2xml = require('js2xml');

/**
 * Generate access token for shop
 *
 * Examples:
 *
 *      insales.token('12345');
 *
 * Output:
 *
 *      8cfa2282b17de0a598c010f5f0109e7d
 * @param {String} token Required, token receive when install app
 * @param {String} secret Required, app secret
 * @returns {String} access token
 */
function token(token, secret) {
  return crypto.createHash('md5')
    .update(token + secret)
    .digest('hex');
}

var options = {
  headers: {
    'Content-Type': 'application/xml'
  },
  xml2js: {
    trim: false,
    explicitArray: false,
    ignoreAttrs: true
  },
  timeout: 15000
};

function error(type, url, msg, response) {
  return {
    type,
    url,
    msg,
    response
  };
}

function output(data, response) {
  return {
    data,
    response
  };
}

function request(r) {
  return new Promise((resolve, reject) => {
    r.once('timeout', ms => {
      reject(error('Timeout', r.url.href, `timeout: ${ms}ms`));
    }).once('error', (err, response) => {
      reject(error('Error', r.url.href, err, response));
    }).once('abort', () => {
      reject(error('Abort', r.url.href, null, null));
    }).once('fail', (data, response) => {
      reject(error('Fail', r.url.href, data, response));
    }).once('success', (data, response) => {
      resolve(output(data, response));
    });
  });
}

const opt = Object.create(options);

function get(conn) {
  if (conn.query) opt.query = conn.query;

  const g = rest.get(`https://${conn.id}:${conn.token}@${conn.url}/admin/${conn.api}.xml`, opt);
  return new Promise((resolve, reject) => {
    request(g)
      .then(resolve)
      .catch(reject);
  });
}

/**
 * GET /admin/account.xml
 *
 * Examples:
 *
 *      insales.getAccount({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru'
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { account:
 *         { blocked: 'false',
 *           city: 'Москва',
 *           'contact-phone': '',
 *           country: 'RU',
 *           'enable-cart-discounts': 'true',
 *           'enable-client-discounts': 'false',
 *           'enable-group-discounts': 'true',
 *           'enable-order-discounts': 'false',
 *           'hide-items-out-of-stock': 'false',
 *           id: '12345',
 *           'notification-email': '',
 *           organization: '',
 *           'sms-notification-phone': '',
 *           state: 'г Москва',
 *           subdomain: 'shop-test',
 *           email: 'mail@mail.ru',
 *           icq: '',
 *           phone: '+71234567890',
 *           title: 'Название магазина',
 *           'minimum-items-price': '',
 *           'stock-currency-exchange-rate': '1.0',
 *           'client-cookies-whitelist': '',
 *           'main-host': 'shop-test.myinsales.ru',
 *           'next-order-number': '1040',
 *           'time-zone': 'Moscow',
 *           owner:
 *            { 'created-at': '2011-11-11 11:11:11 +0400',
 *              email: 'mail@mail.ru',
 *              id: '12345',
 *               name: 'Имя' } } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} id Required, app id
 * @returns {Promise}
 */
function getAccount(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `account`
    }).then(resolve)
      .catch(reject);
  });
}

const opt$1 = Object.create(options);

function post(conn) {
  if (conn.obj !== undefined) {
    const js2xml$$ = new js2xml.Js2Xml(Object.keys(conn.obj)[0], conn.obj[Object.keys(conn.obj)[0]]);
    opt$1.data = js2xml$$.toString();
  }

  const p = rest.post(`https://${conn.id}:${conn.token}@${conn.url}/admin/${conn.api}.xml`, opt$1);
  return new Promise((resolve, reject) => {
    request(p)
      .then(resolve)
      .catch(reject);
  });
}

function getProduct(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `products/${options.productid}`
    }).then(resolve)
      .catch(reject);
  });
}

function createProduct(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: 'products',
      obj: options.product
    }).then(resolve)
      .catch(reject);
  });
}

function listProduct(options, id) {
  const query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;
  if ((options.updated_since) && (options.updated_since !== false)) {
    query.updated_since = options.updated_since;
  }
  if (options.deleted === true) {
    query.deleted = true;
  }

  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: 'products',
      query
    }).then(resolve)
      .catch(reject);
  });
}

function getCollection(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `collections/${options.collectionid}`
    }).then(resolve)
      .catch(reject);
  });
}

function listCollection(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: 'collections'
    }).then(resolve)
      .catch(reject);
  });
}

const opt$2 = Object.create(options);

function remove(conn) {
  const d = rest.del(`https://${conn.id}:${conn.token}@${conn.url}/admin/${conn.api}.xml`, opt$2);
  return new Promise((resolve, reject) => {
    request(d)
      .then(resolve)
      .catch(reject);
  });
}

function getFile(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `files/${options.fileid}`
    }).then(resolve)
      .catch(reject);
  });
}

function createFile(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: 'files',
      obj: options.file
    }).then(resolve)
      .catch(reject);
  });
}

function removeFile(options, id) {
  return new Promise((resolve, reject) => {
    remove({
      id,
      token: options.token,
      url: options.url,
      api: `files/${options.fileid}`
    }).then(resolve)
      .catch(reject);
  });
}

function uploadFile(options, id) {
  const query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;

  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: 'files',
      query
    }).then(resolve)
      .catch(reject);
  });
}

function createJsTag(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: 'js_tags',
      obj: options.jstag
    }).then(resolve)
      .catch(reject);
  });
}

function getRecurring(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `recurring_application_charge`
    }).then(resolve)
      .catch(reject);
  });
}

function createRecurring(options, id) {
  const charge = {
    'recurring-application-charge': {
      'monthly': options.price
    }
  };

  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: 'recurring_application_charge',
      obj: charge
    }).then(resolve)
      .catch(reject);
  });
}

function removeRecurring(options, id) {
  return new Promise((resolve, reject) => {
    remove({
      id,
      token: options.token,
      url: options.url,
      api: `recurring_application_charge`
    }).then(resolve)
      .catch(reject);
  });
}

function getCharge(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `application_charges/${options.chargeid}`
    }).then(resolve)
      .catch(reject);
  });
}

function createCharge(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `application_charges`
    }).then(resolve)
      .catch(reject);
  });
}

function removeCharge(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: 'application_charges',
      obj: options.charge
    }).then(resolve)
      .catch(reject);
  });
}

function listCharge(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: `application_charges/${options.chargeid}/decline`
    }).then(resolve)
      .catch(reject);
  });
}

const opt$3 = Object.create(options);

function put(conn) {
  if (conn.obj !== undefined) {
    const js2xml$$ = new js2xml.Js2Xml(Object.keys(conn.obj)[0], conn.obj[Object.keys(conn.obj)[0]]);
    opt$3.data = js2xml$$.toString();
  }
  const p = rest.put(`https://${conn.id}:${conn.token}@${conn.url}/admin/${conn.api}.xml`, opt$3);
  return new Promise((resolve, reject) => {
    request(p)
      .then(resolve)
      .catch(reject);
  });
}

/**
 * GET /admin/discount_codes/{id}.xml
 *
 * Examples:
 *
 *      insales.getCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        couponid: '1076619'
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-code':
 *         { 'act-once': 'false',
 *           'act-once-for-client': 'false',
 *           code: 'WWWWWW',
 *           'created-at': '2011-11-11T11:11:11+03:00',
 *           description: '',
 *           disabled: 'false',
 *           discount: '10.0',
 *           'expired-at': '2011-11-11',
 *           id: '1111111',
 *           'min-price': '0.0',
 *           'type-id': '1',
 *           'updated-at': '2011-11-11T11:11:11+03:00',
 *           worked: 'true',
 *           'discount-collections': '' } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} options.id Required, coupon id
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function getCoupon(options, id) {
  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `discount_codes/${options.couponid}`
    }).then(resolve)
      .catch(reject);
  });
}

/**
 * GET /admin/discount_codes.xml
 *
 * Examples:
 *
 *      insales.listCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *      }).then(response => {
 *        console.info(response.data);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-codes':
 *         { 'discount-code':
 *            [{ 'act-once': 'false',
 *              'act-once-for-client': 'false',
 *              code: 'WWWWWW',
 *              'created-at': '2011-11-11T11:11:11+03:00',
 *              description: '',
 *              disabled: 'false',
 *              discount: '10.0',
 *              'expired-at': '2011-11-11',
 *              id: '1111111',
 *              'min-price': '0.0',
 *              'type-id': '1',
 *              'updated-at': '2011-11-11T11:11:11+03:00',
 *              worked: 'true',
 *              'discount-collections': '' },
 *            { 'act-once': 'false',
 *              'act-once-for-client': 'false',
 *              code: 'WWWWWY',
 *              'created-at': '2011-11-11T11:11:11+03:00',
 *              description: '',
 *              disabled: 'false',
 *              discount: '10.0',
 *              'expired-at': '2011-11-11',
 *              id: '1111112',
 *              'min-price': '0.0',
 *              'type-id': '1',
 *              'updated-at': '2011-11-11T11:11:11+03:00',
 *              worked: 'true',
 *              'discount-collections': '' }]}}
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} [options.page=1] Page
 * @param {number|string} [options.per_page=250] Coupons per page
 * @param {string} [options.updated_since] Coupons updated since. Format YYYY-MM-DD
 * @param {boolean} [options.deleted] Deleted coupons
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function listCoupon(options, id) {
  const query = {};
  query.page = options.page ? options.page : 1;
  query.per_page = options.per_page ? options.per_page : 250;
  if ((options.updated_since) && (options.updated_since !== false)) {
    options.updated_since = options.updated_since;
  }
  if (options.deleted === true) {
    query.deleted = true;
  }

  return new Promise((resolve, reject) => {
    get({
      id,
      token: options.token,
      url: options.url,
      api: `discount_codes`,
      query
    }).then(resolve)
      .catch(reject);
  });
}

/**
 * POST /admin/discount_codes.xml
 *
 * Examples:
 *
 *      insales.createCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        coupon: {
 *           'discount_code': {
 *             'code': 'frandom2',
 *             'act_once': 'true',
 *             'discount': 10,
 *             'type_id': 1,
 *             'description': 'API code',
 *             'disabled': 0,
 *             'expired-at': '2030-06-09'
 *           }
 *        }
 *      }).then(output => {
 *        console.info(output.response.statusCode);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      { 'discount-code':
 *         { 'act-once': 'true',
 *           'act-once-for-client': '',
 *           code: 'frandom2',
 *           'created-at': '2011-11-11T11:11:11+03:00',
 *           description: 'API code',
 *           disabled: 'false',
 *           discount: '10.0',
 *           'expired-at': '2030-06-09',
 *           id: '100000',
 *           'min-price': '',
 *           'type-id': '1',
 *           'updated-at': '2011-11-11T11:11:11+03:00',
 *           worked: 'true',
 *           'discount-collections': '' } }
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {object} options.coupon Required, coupon
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function createCoupon(options, id) {
  return new Promise((resolve, reject) => {
    post({
      id,
      token: options.token,
      url: options.url,
      api: `discount_codes`,
      obj: options.coupon
    }).then(resolve)
      .catch(reject);
  });
}

/**
 * DELETE /admin/discount_codes/{id}.xml
 *
 * Examples:
 *
 *      insales.removeCoupon({
 *        token: '1234567890',
 *        url: 'shop-test.myinsales.ru',
 *        couponid: '1076619'
 *      }).then(output => {
 *        console.info(output.response.statusCode);
 *      }).catch(err => {
 *        console.error(err);
 *      });
 * Output:
 *
 *      200
 * @param {Object} options
 * @param {string} options.token Required, access token for shop
 * @param {string} options.url Required, shop url
 * @param {number|string} options.couponid Required, coupon id
 * @param {Number|String} id Required, app id
 * @returns {Promise}
 */
function removeCoupon(options, id) {
  return new Promise((resolve, reject) => {
    remove({
      id,
      token: options.token,
      url: options.url,
      api: `discount_codes/${options.couponid}`
    }).then(resolve)
      .catch(reject);
  });
}

function editCoupon(options, id) {
  return new Promise((resolve, reject) => {
    put({
      id,
      token: options.token,
      url: options.url,
      api: `discount_codes/${options.couponid}`,
      obj: options.coupon
    }).then(resolve)
      .catch(reject);
  });
}

class InSales {
  constructor(options) {
    const _options = options || {};
    if (!_options.id) throw new Error('Missing app id');
    if (!_options.secret) throw new Error('Missing app secret');

    this.id = _options.id;
    this.secret = _options.secret;
  }

  token(tok) {
    return token(tok, this.secret);
  }

  getProduct(options) {
    return getProduct(options, this.id);
  }

  createProduct(options) {
    return createProduct(options, this.id);
  }

  listProduct(options) {
    return listProduct(options, this.id);
  }

  getCollection(options) {
    return getCollection(options, this.id);
  }

  listCollection(options) {
    return listCollection(options, this.id);
  }

  getFile(options) {
    return getFile(options, this.id);
  }

  createFile(options) {
    return createFile(options, this.id);
  }

  removeFile(options) {
    return removeFile(options, this.id);
  }

  uploadFile(options) {
    return uploadFile(options, this.id);
  }

  getAccount(options) {
    return getAccount(options, this.id);
  }

  createJsTag(options) {
    return createJsTag(options, this.id);
  }

  getRecurring(options) {
    return getRecurring(options, this.id);
  }

  createRecurring(options) {
    return createRecurring(options, this.id);
  }

  removeRecurring(options) {
    return removeRecurring(options, this.id);
  }

  getCharge(options) {
    return getCharge(options, this.id);
  }

  createCharge(options) {
    return createCharge(options, this.id);
  }

  removeCharge(options) {
    return removeCharge(options, this.id);
  }

  listCharge(options) {
    return listCharge(options, this.id);
  }

  getCoupon(options) {
    return getCoupon(options, this.id);
  }

  createCoupon(options) {
    return createCoupon(options, this.id);
  }

  removeCoupon(options) {
    return removeCoupon(options, this.id);
  }

  listCoupon(options) {
    return listCoupon(options, this.id);
  }

  editCoupon(options) {
    return editCoupon(options, this.id);
  }
}

var index = options => new InSales(options);

module.exports = index;
